﻿using Prism.Commands;
using Prism.Mvvm;

namespace GridSpanSample.ViewModels
{
    /// <summary>
    /// Usage:
    /// 1. Bind With and Height in Page like:
    ///     ContentPage xmlns="http://xamarin.com/schemas/2014/forms" 
    ///         ...
    ///         Width="{Binding Path=PageWidth, Mode=OneWayToSource}"
    ///         Height="{Binding Path=PageHeight, Mode=OneWayToSource}" >
    /// 
    /// 2. Bind to the PageSizeChangedCommand via Page behavior like:
    ///     ContentPage.Behaviors>
    ///                behaviors:EventToCommandBehavior EventName = "SizeChanged" Command="{Binding PageSizeChangedCommand}" />
    ///     /ContentPage.Behaviors>
    /// 
    /// 3. Use IsPortraitMode or IsLandscapeMode in XAML or Code
    /// 4. and/or implement OnPortraitMode() and/or OnLandscapeMode()
    /// 
    /// </summary>
    public abstract class OrientationAwareBase : BindableBase
    {
        private bool _isPortraitMode;
        private bool _isLandscapeMode;

        /// <summary>
        /// Could not be set in ViewModel
        /// </summary>
        public int PageWidth { get; set; }

        public int PageHeight { get; set; }

        public bool IsPortraitMode
        {
            get { return _isPortraitMode; }
            set { this.SetProperty(ref _isPortraitMode, value); }
        }

        public bool IsLandscapeMode
        {
            get { return _isLandscapeMode; }
            set { this.SetProperty(ref _isLandscapeMode, value); }
        }


        public DelegateCommand PageSizeChangedCommand => new DelegateCommand(this.OnPageSizeChanged);

        private void OnPageSizeChanged()
        {
            // PortraitMode
            if (PageWidth < PageHeight)
            {
                this.IsPortraitMode = true;
                this.IsLandscapeMode = false;

                this.OnPortraitMode();
            }
            // LandscapeMode
            else
            {
                this.IsPortraitMode = false;
                this.IsLandscapeMode = true;

                this.OnLandscapeMode();
            }
        }

        public abstract void OnPortraitMode();

        public abstract void OnLandscapeMode();
    }
}